#!/usr/bin/env python
# Working 09-08-2020

'''
Asynchronous Modbus Server Built in Python using the pyModbus module
'''

# Import the libraries we need
#from pymodbus.server.async import StartTcpServer
from pymodbus.server.asynchronous import StartTcpServer
from pymodbus.device import ModbusDeviceIdentification
from pymodbus.datastore import ModbusSequentialDataBlock
from pymodbus.datastore import ModbusSlaveContext, ModbusServerContext
# Added for plc stuff
import plc_sensor_pump
import os
from pymodbus.client.sync import ModbusTcpClient

# Global variables
HOST = '127.0.0.1'
client = None

def initialize():
    global client
    Power=True
    SP_Start=50
    SP_Stop=75
    Tank_Level=25
    
    client = ModbusTcpClient(HOST)
     # Set registers
    address=1
    for value in (Power, False, False, False, False):
        plc_sensor_pump.write_data(address, value, client)  
        address += 1
    address=400001
    value=Tank_Level
    plc_sensor_pump.write_data(address, value, client)
    address=400002
    value=SP_Stop
    plc_sensor_pump.write_data(address, value, client)
    address=400003
    value=SP_Start
    plc_sensor_pump.write_data(address, value, client)   
    
def scan():
    # Check reset
    if plc_sensor_pump.read_data(5, 1, client)[0]:
        initialize()
    # Check power status
    elif plc_sensor_pump.read_data(1, 1, client)[0]:
        # Check stop
        if plc_sensor_pump.read_data(2,1, client)[0]:
            # Turn off pump
            plc_sensor_pump.write_data(4, False, client)
            # Turn off power
            plc_sensor_pump.write_data(1, False, client)
            # Reset stop (Needed for Advanced HMI)
            plc_sensor_pump.write_data(2, False, client)
        else:
            tank_data = plc_sensor_pump.read_data(400001, 3, client)
            if tank_data[0] < tank_data[2]:
                # Turn on pump
                plc_sensor_pump.write_data(4, True, client)
            elif tank_data[0] > tank_data[1]:
                # Turn off pump
                plc_sensor_pump.write_data(4, False, client)
            
    # Check power switch
    elif plc_sensor_pump.read_data(3, 1, client)[0]:
        # Turn on power
        plc_sensor_pump.write_data(1, True, client)


def plc():
    if not plc_sensor_pump.check_host(HOST, 2, 1):
        exit("No PLC runnning")
    else:
        print("Connected to PLC")
        
    initialize()
    while True:
        scan()

# Create a datastore and populate it with test data
store = ModbusSlaveContext(
   di = ModbusSequentialDataBlock(0, [17]*100),    # Discrete Inputs initializer
   co = ModbusSequentialDataBlock(0, [17]*100),    # Coils initializer
   hr = ModbusSequentialDataBlock(0, [17]*100),    # Holding Register initializer
   ir = ModbusSequentialDataBlock(0, [17]*100))    # Input Registers initializer

context = ModbusServerContext(slaves=store, single=True)

# Populate the Modbus server information fields, these get returned as
#  response to identity queries
identity = ModbusDeviceIdentification()
identity.VendorName  = 'PyModbus Inc.'
identity.ProductCode = 'PM'
identity.VendorUrl   = 'https://github.com/riptideio/pyModbus'
identity.ProductName = 'Modbus Server'
identity.ModelName   = 'PyModbus'
identity.MajorMinorRevision = '1.0'


newpid = os.fork()
if newpid == 0:
    plc()
else:
    # Start the listening server
    print("Starting Modbus server...")
    StartTcpServer(context, identity=identity, address=("0.0.0.0", 502))


